@@ -17,6 +17,8 @@
 package handler_test
 
 import (
+	"context"
+
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	appsv1 "k8s.io/api/apps/v1"
@@ -26,30 +28,34 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
+	"k8s.io/client-go/rest"
 	"k8s.io/client-go/util/workqueue"
+	"k8s.io/utils/ptr"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/apiutil"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllertest"
+	"sigs.k8s.io/controller-runtime/pkg/controller/priorityqueue"
 	"sigs.k8s.io/controller-runtime/pkg/event"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 )
 
 var _ = Describe("Eventhandler", func() {
-	var q workqueue.RateLimitingInterface
+	var ctx = context.Background()
+	var q workqueue.TypedRateLimitingInterface[reconcile.Request]
 	var instance handler.EnqueueRequestForObject
 	var pod *corev1.Pod
 	var mapper meta.RESTMapper
-	t := true
 	BeforeEach(func() {
-		q = controllertest.Queue{Interface: workqueue.New()}
+		q = &controllertest.Queue{TypedInterface: workqueue.NewTyped[reconcile.Request]()}
 		pod = &corev1.Pod{
 			ObjectMeta: metav1.ObjectMeta{Namespace: "biz", Name: "baz"},
 		}
 		Expect(cfg).NotTo(BeNil())
 
-		var err error
-		mapper, err = apiutil.NewDiscoveryRESTMapper(cfg)
+		httpClient, err := rest.HTTPClientFor(cfg)
+		Expect(err).ShouldNot(HaveOccurred())
+		mapper, err = apiutil.NewDynamicRESTMapper(cfg, httpClient)
 		Expect(err).ShouldNot(HaveOccurred())
 	})
 
@@ -58,13 +64,10 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
-			i, _ := q.Get()
-			Expect(i).NotTo(BeNil())
-			req, ok := i.(reconcile.Request)
-			Expect(ok).To(BeTrue())
+			req, _ := q.Get()
 			Expect(req.NamespacedName).To(Equal(types.NamespacedName{Namespace: "biz", Name: "baz"}))
 		})
 
@@ -72,13 +75,10 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.DeleteEvent{
 				Object: pod,
 			}
-			instance.Delete(evt, q)
+			instance.Delete(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
-			i, _ := q.Get()
-			Expect(i).NotTo(BeNil())
-			req, ok := i.(reconcile.Request)
-			Expect(ok).To(BeTrue())
+			req, _ := q.Get()
 			Expect(req.NamespacedName).To(Equal(types.NamespacedName{Namespace: "biz", Name: "baz"}))
 		})
 
@@ -92,13 +92,10 @@ var _ = Describe("Eventhandler", func() {
 					ObjectOld: pod,
 					ObjectNew: newPod,
 				}
-				instance.Update(evt, q)
+				instance.Update(ctx, evt, q)
 				Expect(q.Len()).To(Equal(1))
 
-				i, _ := q.Get()
-				Expect(i).NotTo(BeNil())
-				req, ok := i.(reconcile.Request)
-				Expect(ok).To(BeTrue())
+				req, _ := q.Get()
 				Expect(req.NamespacedName).To(Equal(types.NamespacedName{Namespace: "biz2", Name: "baz2"}))
 			})
 
@@ -106,12 +103,9 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.GenericEvent{
 				Object: pod,
 			}
-			instance.Generic(evt, q)
+			instance.Generic(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
-			i, _ := q.Get()
-			Expect(i).NotTo(BeNil())
-			req, ok := i.(reconcile.Request)
-			Expect(ok).To(BeTrue())
+			req, _ := q.Get()
 			Expect(req.NamespacedName).To(Equal(types.NamespacedName{Namespace: "biz", Name: "baz"}))
 		})
 
@@ -120,7 +114,7 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.CreateEvent{
 					Object: nil,
 				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 
@@ -133,22 +127,16 @@ var _ = Describe("Eventhandler", func() {
 					ObjectNew: newPod,
 					ObjectOld: nil,
 				}
-				instance.Update(evt, q)
+				instance.Update(ctx, evt, q)
 				Expect(q.Len()).To(Equal(1))
-				i, _ := q.Get()
-				Expect(i).NotTo(BeNil())
-				req, ok := i.(reconcile.Request)
-				Expect(ok).To(BeTrue())
+				req, _ := q.Get()
 				Expect(req.NamespacedName).To(Equal(types.NamespacedName{Namespace: "biz2", Name: "baz2"}))
 
 				evt.ObjectNew = nil
 				evt.ObjectOld = pod
-				instance.Update(evt, q)
+				instance.Update(ctx, evt, q)
 				Expect(q.Len()).To(Equal(1))
-				i, _ = q.Get()
-				Expect(i).NotTo(BeNil())
-				req, ok = i.(reconcile.Request)
-				Expect(ok).To(BeTrue())
+				req, _ = q.Get()
 				Expect(req.NamespacedName).To(Equal(types.NamespacedName{Namespace: "biz", Name: "baz"}))
 			})
 
@@ -156,7 +144,7 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.DeleteEvent{
 					Object: nil,
 				}
-				instance.Delete(evt, q)
+				instance.Delete(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 
@@ -164,7 +152,7 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.GenericEvent{
 					Object: nil,
 				}
-				instance.Generic(evt, q)
+				instance.Generic(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 		})
@@ -173,7 +161,7 @@ var _ = Describe("Eventhandler", func() {
 	Describe("EnqueueRequestsFromMapFunc", func() {
 		It("should enqueue a Request with the function applied to the CreateEvent.", func() {
 			req := []reconcile.Request{}
-			instance := handler.EnqueueRequestsFromMapFunc(func(a client.Object) []reconcile.Request {
+			instance := handler.EnqueueRequestsFromMapFunc(func(ctx context.Context, a client.Object) []reconcile.Request {
 				defer GinkgoRecover()
 				Expect(a).To(Equal(pod))
 				req = []reconcile.Request{
@@ -190,7 +178,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(2))
 
 			i1, _ := q.Get()
@@ -205,7 +193,7 @@ var _ = Describe("Eventhandler", func() {
 
 		It("should enqueue a Request with the function applied to the DeleteEvent.", func() {
 			req := []reconcile.Request{}
-			instance := handler.EnqueueRequestsFromMapFunc(func(a client.Object) []reconcile.Request {
+			instance := handler.EnqueueRequestsFromMapFunc(func(ctx context.Context, a client.Object) []reconcile.Request {
 				defer GinkgoRecover()
 				Expect(a).To(Equal(pod))
 				req = []reconcile.Request{
@@ -222,7 +210,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.DeleteEvent{
 				Object: pod,
 			}
-			instance.Delete(evt, q)
+			instance.Delete(ctx, evt, q)
 			Expect(q.Len()).To(Equal(2))
 
 			i1, _ := q.Get()
@@ -241,7 +229,7 @@ var _ = Describe("Eventhandler", func() {
 
 				req := []reconcile.Request{}
 
-				instance := handler.EnqueueRequestsFromMapFunc(func(a client.Object) []reconcile.Request {
+				instance := handler.EnqueueRequestsFromMapFunc(func(ctx context.Context, a client.Object) []reconcile.Request {
 					defer GinkgoRecover()
 					req = []reconcile.Request{
 						{
@@ -258,7 +246,7 @@ var _ = Describe("Eventhandler", func() {
 					ObjectOld: pod,
 					ObjectNew: newPod,
 				}
-				instance.Update(evt, q)
+				instance.Update(ctx, evt, q)
 				Expect(q.Len()).To(Equal(2))
 
 				i, _ := q.Get()
@@ -270,7 +258,7 @@ var _ = Describe("Eventhandler", func() {
 
 		It("should enqueue a Request with the function applied to the GenericEvent.", func() {
 			req := []reconcile.Request{}
-			instance := handler.EnqueueRequestsFromMapFunc(func(a client.Object) []reconcile.Request {
+			instance := handler.EnqueueRequestsFromMapFunc(func(ctx context.Context, a client.Object) []reconcile.Request {
 				defer GinkgoRecover()
 				Expect(a).To(Equal(pod))
 				req = []reconcile.Request{
@@ -287,7 +275,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.GenericEvent{
 				Object: pod,
 			}
-			instance.Generic(evt, q)
+			instance.Generic(ctx, evt, q)
 			Expect(q.Len()).To(Equal(2))
 
 			i1, _ := q.Get()
@@ -303,11 +291,7 @@ var _ = Describe("Eventhandler", func() {
 
 	Describe("EnqueueRequestForOwner", func() {
 		It("should enqueue a Request with the Owner of the object in the CreateEvent.", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &appsv1.ReplicaSet{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
@@ -319,7 +303,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
 			i, _ := q.Get()
@@ -328,11 +312,7 @@ var _ = Describe("Eventhandler", func() {
 		})
 
 		It("should enqueue a Request with the Owner of the object in the DeleteEvent.", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &appsv1.ReplicaSet{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
@@ -344,7 +324,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.DeleteEvent{
 				Object: pod,
 			}
-			instance.Delete(evt, q)
+			instance.Delete(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
 			i, _ := q.Get()
@@ -357,11 +337,7 @@ var _ = Describe("Eventhandler", func() {
 			newPod.Name = pod.Name + "2"
 			newPod.Namespace = pod.Namespace + "2"
 
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &appsv1.ReplicaSet{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
@@ -381,7 +357,7 @@ var _ = Describe("Eventhandler", func() {
 				ObjectOld: pod,
 				ObjectNew: newPod,
 			}
-			instance.Update(evt, q)
+			instance.Update(ctx, evt, q)
 			Expect(q.Len()).To(Equal(2))
 
 			i1, _ := q.Get()
@@ -398,11 +374,7 @@ var _ = Describe("Eventhandler", func() {
 			newPod := pod.DeepCopy()
 			newPod.Name = pod.Name + "2"
 
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &appsv1.ReplicaSet{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
@@ -422,7 +394,7 @@ var _ = Describe("Eventhandler", func() {
 				ObjectOld: pod,
 				ObjectNew: newPod,
 			}
-			instance.Update(evt, q)
+			instance.Update(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
 			i, _ := q.Get()
@@ -431,12 +403,7 @@ var _ = Describe("Eventhandler", func() {
 		})
 
 		It("should enqueue a Request with the Owner of the object in the GenericEvent.", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &appsv1.ReplicaSet{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
-
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
 					Name:       "foo-parent",
@@ -447,7 +414,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.GenericEvent{
 				Object: pod,
 			}
-			instance.Generic(evt, q)
+			instance.Generic(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
 			i, _ := q.Get()
@@ -456,12 +423,7 @@ var _ = Describe("Eventhandler", func() {
 		})
 
 		It("should not enqueue a Request if there are no owners matching Group and Kind.", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType:    &appsv1.ReplicaSet{},
-				IsController: t,
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{}, handler.OnlyControllerOwner())
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{ // Wrong group
 					Name:       "foo1-parent",
@@ -477,28 +439,24 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(0))
 		})
 
 		It("should enqueue a Request if there are owners matching Group "+
 			"and Kind with a different version.", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &autoscalingv1.HorizontalPodAutoscaler{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &autoscalingv1.HorizontalPodAutoscaler{})
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
 					Name:       "foo-parent",
 					Kind:       "HorizontalPodAutoscaler",
-					APIVersion: "autoscaling/v2beta1",
+					APIVersion: "autoscaling/v2",
 				},
 			}
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
 			i, _ := q.Get()
@@ -507,11 +465,7 @@ var _ = Describe("Eventhandler", func() {
 		})
 
 		It("should enqueue a Request for a owner that is cluster scoped", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &corev1.Node{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &corev1.Node{})
 			pod.OwnerReferences = []metav1.OwnerReference{
 				{
 					Name:       "node-1",
@@ -522,7 +476,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(1))
 
 			i, _ := q.Get()
@@ -532,27 +486,18 @@ var _ = Describe("Eventhandler", func() {
 		})
 
 		It("should not enqueue a Request if there are no owners.", func() {
-			instance := handler.EnqueueRequestForOwner{
-				OwnerType: &appsv1.ReplicaSet{},
-			}
-			Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-			Expect(instance.InjectMapper(mapper)).To(Succeed())
+			instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 			Expect(q.Len()).To(Equal(0))
 		})
 
 		Context("with the Controller field set to true", func() {
 			It("should enqueue reconcile.Requests for only the first the Controller if there are "+
 				"multiple Controller owners.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType:    &appsv1.ReplicaSet{},
-					IsController: t,
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
+				instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{}, handler.OnlyControllerOwner())
 				pod.OwnerReferences = []metav1.OwnerReference{
 					{
 						Name:       "foo1-parent",
@@ -563,7 +508,7 @@ var _ = Describe("Eventhandler", func() {
 						Name:       "foo2-parent",
 						Kind:       "ReplicaSet",
 						APIVersion: "apps/v1",
-						Controller: &t,
+						Controller: ptr.To(true),
 					},
 					{
 						Name:       "foo3-parent",
@@ -574,7 +519,7 @@ var _ = Describe("Eventhandler", func() {
 						Name:       "foo4-parent",
 						Kind:       "ReplicaSet",
 						APIVersion: "apps/v1",
-						Controller: &t,
+						Controller: ptr.To(true),
 					},
 					{
 						Name:       "foo5-parent",
@@ -585,7 +530,7 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.CreateEvent{
 					Object: pod,
 				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(1))
 				i, _ := q.Get()
 				Expect(i).To(Equal(reconcile.Request{
@@ -593,12 +538,7 @@ var _ = Describe("Eventhandler", func() {
 			})
 
 			It("should not enqueue reconcile.Requests if there are no Controller owners.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType:    &appsv1.ReplicaSet{},
-					IsController: t,
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
+				instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{}, handler.OnlyControllerOwner())
 				pod.OwnerReferences = []metav1.OwnerReference{
 					{
 						Name:       "foo1-parent",
@@ -619,32 +559,23 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.CreateEvent{
 					Object: pod,
 				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 
 			It("should not enqueue reconcile.Requests if there are no owners.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType:    &appsv1.ReplicaSet{},
-					IsController: t,
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
+				instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{}, handler.OnlyControllerOwner())
 				evt := event.CreateEvent{
 					Object: pod,
 				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 		})
 
 		Context("with the Controller field set to false", func() {
 			It("should enqueue a reconcile.Requests for all owners.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType: &appsv1.ReplicaSet{},
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
+				instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 				pod.OwnerReferences = []metav1.OwnerReference{
 					{
 						Name:       "foo1-parent",
@@ -665,7 +596,7 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.CreateEvent{
 					Object: pod,
 				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(3))
 
 				i1, _ := q.Get()
@@ -684,11 +615,7 @@ var _ = Describe("Eventhandler", func() {
 
 		Context("with a nil object", func() {
 			It("should do nothing.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType: &appsv1.ReplicaSet{},
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
+				instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 				pod.OwnerReferences = []metav1.OwnerReference{
 					{
 						Name:       "foo1-parent",
@@ -699,81 +626,22 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.CreateEvent{
 					Object: nil,
 				}
-				instance.Create(evt, q)
-				Expect(q.Len()).To(Equal(0))
-			})
-		})
-
-		Context("with a multiple matching kinds", func() {
-			It("should do nothing.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType: &metav1.ListOptions{},
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).NotTo(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
-				pod.OwnerReferences = []metav1.OwnerReference{
-					{
-						Name:       "foo1-parent",
-						Kind:       "ListOptions",
-						APIVersion: "meta/v1",
-					},
-				}
-				evt := event.CreateEvent{
-					Object: pod,
-				}
-				instance.Create(evt, q)
-				Expect(q.Len()).To(Equal(0))
-			})
-		})
-		Context("with an OwnerType that cannot be resolved", func() {
-			It("should do nothing.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType: &controllertest.ErrorType{},
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).NotTo(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
-				pod.OwnerReferences = []metav1.OwnerReference{
-					{
-						Name:       "foo1-parent",
-						Kind:       "ListOptions",
-						APIVersion: "meta/v1",
-					},
-				}
-				evt := event.CreateEvent{
-					Object: pod,
-				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 		})
 
 		Context("with a nil OwnerType", func() {
-			It("should do nothing.", func() {
-				instance := handler.EnqueueRequestForOwner{}
-				Expect(instance.InjectScheme(scheme.Scheme)).NotTo(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
-				pod.OwnerReferences = []metav1.OwnerReference{
-					{
-						Name:       "foo1-parent",
-						Kind:       "OwnerType",
-						APIVersion: "meta/v1",
-					},
-				}
-				evt := event.CreateEvent{
-					Object: pod,
-				}
-				instance.Create(evt, q)
-				Expect(q.Len()).To(Equal(0))
+			It("should panic", func() {
+				Expect(func() {
+					handler.EnqueueRequestForOwner(nil, nil, nil)
+				}).To(Panic())
 			})
 		})
 
 		Context("with an invalid APIVersion in the OwnerReference", func() {
 			It("should do nothing.", func() {
-				instance := handler.EnqueueRequestForOwner{
-					OwnerType: &appsv1.ReplicaSet{},
-				}
-				Expect(instance.InjectScheme(scheme.Scheme)).To(Succeed())
-				Expect(instance.InjectMapper(mapper)).To(Succeed())
+				instance := handler.EnqueueRequestForOwner(scheme.Scheme, mapper, &appsv1.ReplicaSet{})
 				pod.OwnerReferences = []metav1.OwnerReference{
 					{
 						Name:       "foo1-parent",
@@ -784,27 +652,27 @@ var _ = Describe("Eventhandler", func() {
 				evt := event.CreateEvent{
 					Object: pod,
 				}
-				instance.Create(evt, q)
+				instance.Create(ctx, evt, q)
 				Expect(q.Len()).To(Equal(0))
 			})
 		})
 	})
 
 	Describe("Funcs", func() {
-		failingFuncs := handler.Funcs{
-			CreateFunc: func(event.CreateEvent, workqueue.RateLimitingInterface) {
+		failingFuncs := handler.TypedFuncs[client.Object, reconcile.Request]{
+			CreateFunc: func(context.Context, event.CreateEvent, workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Fail("Did not expect CreateEvent to be called.")
 			},
-			DeleteFunc: func(event.DeleteEvent, workqueue.RateLimitingInterface) {
+			DeleteFunc: func(context.Context, event.DeleteEvent, workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Fail("Did not expect DeleteEvent to be called.")
 			},
-			UpdateFunc: func(event.UpdateEvent, workqueue.RateLimitingInterface) {
+			UpdateFunc: func(context.Context, event.UpdateEvent, workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Fail("Did not expect UpdateEvent to be called.")
 			},
-			GenericFunc: func(event.GenericEvent, workqueue.RateLimitingInterface) {
+			GenericFunc: func(context.Context, event.GenericEvent, workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Fail("Did not expect GenericEvent to be called.")
 			},
@@ -815,12 +683,12 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.CreateFunc = func(evt2 event.CreateEvent, q2 workqueue.RateLimitingInterface) {
+			instance.CreateFunc = func(ctx context.Context, evt2 event.CreateEvent, q2 workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Expect(q2).To(Equal(q))
 				Expect(evt2).To(Equal(evt))
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 		})
 
 		It("should NOT call CreateFunc for a CreateEvent if NOT provided.", func() {
@@ -829,7 +697,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.CreateEvent{
 				Object: pod,
 			}
-			instance.Create(evt, q)
+			instance.Create(ctx, evt, q)
 		})
 
 		It("should call UpdateFunc for an UpdateEvent if provided.", func() {
@@ -842,13 +710,13 @@ var _ = Describe("Eventhandler", func() {
 			}
 
 			instance := failingFuncs
-			instance.UpdateFunc = func(evt2 event.UpdateEvent, q2 workqueue.RateLimitingInterface) {
+			instance.UpdateFunc = func(ctx context.Context, evt2 event.UpdateEvent, q2 workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Expect(q2).To(Equal(q))
 				Expect(evt2).To(Equal(evt))
 			}
 
-			instance.Update(evt, q)
+			instance.Update(ctx, evt, q)
 		})
 
 		It("should NOT call UpdateFunc for an UpdateEvent if NOT provided.", func() {
@@ -859,7 +727,7 @@ var _ = Describe("Eventhandler", func() {
 				ObjectOld: pod,
 				ObjectNew: newPod,
 			}
-			instance.Update(evt, q)
+			instance.Update(ctx, evt, q)
 		})
 
 		It("should call DeleteFunc for a DeleteEvent if provided.", func() {
@@ -867,12 +735,12 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.DeleteEvent{
 				Object: pod,
 			}
-			instance.DeleteFunc = func(evt2 event.DeleteEvent, q2 workqueue.RateLimitingInterface) {
+			instance.DeleteFunc = func(ctx context.Context, evt2 event.DeleteEvent, q2 workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Expect(q2).To(Equal(q))
 				Expect(evt2).To(Equal(evt))
 			}
-			instance.Delete(evt, q)
+			instance.Delete(ctx, evt, q)
 		})
 
 		It("should NOT call DeleteFunc for a DeleteEvent if NOT provided.", func() {
@@ -881,7 +749,7 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.DeleteEvent{
 				Object: pod,
 			}
-			instance.Delete(evt, q)
+			instance.Delete(ctx, evt, q)
 		})
 
 		It("should call GenericFunc for a GenericEvent if provided.", func() {
@@ -889,12 +757,12 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.GenericEvent{
 				Object: pod,
 			}
-			instance.GenericFunc = func(evt2 event.GenericEvent, q2 workqueue.RateLimitingInterface) {
+			instance.GenericFunc = func(ctx context.Context, evt2 event.GenericEvent, q2 workqueue.TypedRateLimitingInterface[reconcile.Request]) {
 				defer GinkgoRecover()
 				Expect(q2).To(Equal(q))
 				Expect(evt2).To(Equal(evt))
 			}
-			instance.Generic(evt, q)
+			instance.Generic(ctx, evt, q)
 		})
 
 		It("should NOT call GenericFunc for a GenericEvent if NOT provided.", func() {
@@ -903,7 +771,275 @@ var _ = Describe("Eventhandler", func() {
 			evt := event.GenericEvent{
 				Object: pod,
 			}
-			instance.Generic(evt, q)
+			instance.Generic(ctx, evt, q)
 		})
 	})
+
+	Describe("WithLowPriorityWhenUnchanged", func() {
+		handlerPriorityTests := []struct {
+			name    string
+			handler func() handler.EventHandler
+		}{
+			{
+				name:    "WithLowPriorityWhenUnchanged wrapper",
+				handler: func() handler.EventHandler { return handler.WithLowPriorityWhenUnchanged(customHandler{}) },
+			},
+			{
+				name:    "EnqueueRequestForObject",
+				handler: func() handler.EventHandler { return &handler.EnqueueRequestForObject{} },
+			},
+			{
+				name: "EnqueueRequestForOwner",
+				handler: func() handler.EventHandler {
+					return handler.EnqueueRequestForOwner(
+						scheme.Scheme,
+						mapper,
+						&corev1.Pod{},
+					)
+				},
+			},
+			{
+				name: "TypedEnqueueRequestForOwner",
+				handler: func() handler.EventHandler {
+					return handler.TypedEnqueueRequestForOwner[client.Object](
+						scheme.Scheme,
+						mapper,
+						&corev1.Pod{},
+					)
+				},
+			},
+			{
+				name: "Funcs",
+				handler: func() handler.EventHandler {
+					return handler.TypedFuncs[client.Object, reconcile.Request]{
+						CreateFunc: func(ctx context.Context, tce event.TypedCreateEvent[client.Object], wq workqueue.TypedRateLimitingInterface[reconcile.Request]) {
+							wq.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+								Namespace: tce.Object.GetNamespace(),
+								Name:      tce.Object.GetName(),
+							}})
+						},
+						UpdateFunc: func(ctx context.Context, tue event.TypedUpdateEvent[client.Object], wq workqueue.TypedRateLimitingInterface[reconcile.Request]) {
+							wq.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+								Namespace: tue.ObjectNew.GetNamespace(),
+								Name:      tue.ObjectNew.GetName(),
+							}})
+						},
+					}
+				},
+			},
+			{
+				name: "EnqueueRequestsFromMapFunc",
+				handler: func() handler.EventHandler {
+					return handler.EnqueueRequestsFromMapFunc(func(ctx context.Context, obj client.Object) []reconcile.Request {
+						return []reconcile.Request{{NamespacedName: types.NamespacedName{
+							Name:      obj.GetName(),
+							Namespace: obj.GetNamespace(),
+						}}}
+					})
+				},
+			},
+		}
+		for _, test := range handlerPriorityTests {
+			When("handler is "+test.name, func() {
+				It("should lower the priority of a create request for an object that was part of the initial list", func() {
+					actualOpts := priorityqueue.AddOpts{}
+					var actualRequests []reconcile.Request
+					wq := &fakePriorityQueue{
+						addWithOpts: func(o priorityqueue.AddOpts, items ...reconcile.Request) {
+							actualOpts = o
+							actualRequests = items
+						},
+					}
+
+					test.handler().Create(ctx, event.CreateEvent{
+						Object: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name:              "my-pod",
+							CreationTimestamp: metav1.Now(),
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+						IsInInitialList: true,
+					}, wq)
+
+					Expect(actualOpts).To(Equal(priorityqueue.AddOpts{Priority: handler.LowPriority}))
+					Expect(actualRequests).To(Equal([]reconcile.Request{{NamespacedName: types.NamespacedName{Name: "my-pod"}}}))
+				})
+
+				It("should not lower the priority of a create request for an object that was not part of the initial list", func() {
+					actualOpts := priorityqueue.AddOpts{}
+					var actualRequests []reconcile.Request
+					wq := &fakePriorityQueue{
+						addWithOpts: func(o priorityqueue.AddOpts, items ...reconcile.Request) {
+							actualOpts = o
+							actualRequests = items
+						},
+					}
+
+					test.handler().Create(ctx, event.CreateEvent{
+						Object: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name:              "my-pod",
+							CreationTimestamp: metav1.Now(),
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+						IsInInitialList: false,
+					}, wq)
+
+					Expect(actualOpts).To(Equal(priorityqueue.AddOpts{}))
+					Expect(actualRequests).To(Equal([]reconcile.Request{{NamespacedName: types.NamespacedName{Name: "my-pod"}}}))
+				})
+
+				It("should lower the priority of an update request with unchanged RV", func() {
+					actualOpts := priorityqueue.AddOpts{}
+					var actualRequests []reconcile.Request
+					wq := &fakePriorityQueue{
+						addWithOpts: func(o priorityqueue.AddOpts, items ...reconcile.Request) {
+							actualOpts = o
+							actualRequests = items
+						},
+					}
+
+					test.handler().Update(ctx, event.UpdateEvent{
+						ObjectOld: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name: "my-pod",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+						ObjectNew: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name: "my-pod",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+					}, wq)
+
+					Expect(actualOpts).To(Equal(priorityqueue.AddOpts{Priority: handler.LowPriority}))
+					Expect(actualRequests).To(Equal([]reconcile.Request{{NamespacedName: types.NamespacedName{Name: "my-pod"}}}))
+				})
+
+				It("should not lower the priority of an update request with changed RV", func() {
+					actualOpts := priorityqueue.AddOpts{}
+					var actualRequests []reconcile.Request
+					wq := &fakePriorityQueue{
+						addWithOpts: func(o priorityqueue.AddOpts, items ...reconcile.Request) {
+							actualOpts = o
+							actualRequests = items
+						},
+					}
+
+					test.handler().Update(ctx, event.UpdateEvent{
+						ObjectOld: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name: "my-pod",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+						ObjectNew: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name:            "my-pod",
+							ResourceVersion: "1",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+					}, wq)
+
+					Expect(actualOpts).To(Equal(priorityqueue.AddOpts{}))
+					Expect(actualRequests).To(Equal([]reconcile.Request{{NamespacedName: types.NamespacedName{Name: "my-pod"}}}))
+				})
+
+				It("should have no effect on create if the workqueue is not a priorityqueue", func() {
+					test.handler().Create(ctx, event.CreateEvent{
+						Object: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name: "my-pod",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+					}, q)
+
+					Expect(q.Len()).To(Equal(1))
+					item, _ := q.Get()
+					Expect(item).To(Equal(reconcile.Request{NamespacedName: types.NamespacedName{Name: "my-pod"}}))
+				})
+
+				It("should have no effect on Update if the workqueue is not a priorityqueue", func() {
+					test.handler().Update(ctx, event.UpdateEvent{
+						ObjectOld: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name: "my-pod",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+						ObjectNew: &corev1.Pod{ObjectMeta: metav1.ObjectMeta{
+							Name: "my-pod",
+							OwnerReferences: []metav1.OwnerReference{{
+								Kind: "Pod",
+								Name: "my-pod",
+							}},
+						}},
+					}, q)
+
+					Expect(q.Len()).To(Equal(1))
+					item, _ := q.Get()
+					Expect(item).To(Equal(reconcile.Request{NamespacedName: types.NamespacedName{Name: "my-pod"}}))
+				})
+			})
+		}
+	})
 })
+
+type fakePriorityQueue struct {
+	workqueue.TypedRateLimitingInterface[reconcile.Request]
+	addWithOpts func(o priorityqueue.AddOpts, items ...reconcile.Request)
+}
+
+func (f *fakePriorityQueue) Add(item reconcile.Request) {
+	f.AddWithOpts(priorityqueue.AddOpts{}, item)
+}
+
+func (f *fakePriorityQueue) AddWithOpts(o priorityqueue.AddOpts, items ...reconcile.Request) {
+	f.addWithOpts(o, items...)
+}
+func (f *fakePriorityQueue) GetWithPriority() (item reconcile.Request, priority int, shutdown bool) {
+	panic("GetWithPriority is not expected to be called")
+}
+
+// customHandler re-implements the basic enqueueRequestForObject logic
+// to be able to test the WithLowPriorityWhenUnchanged wrapper
+type customHandler struct{}
+
+func (ch customHandler) Create(ctx context.Context, evt event.CreateEvent, q workqueue.TypedRateLimitingInterface[reconcile.Request]) {
+	q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+		Namespace: evt.Object.GetNamespace(),
+		Name:      evt.Object.GetName(),
+	}})
+}
+func (ch customHandler) Update(ctx context.Context, evt event.UpdateEvent, q workqueue.TypedRateLimitingInterface[reconcile.Request]) {
+	q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+		Namespace: evt.ObjectNew.GetNamespace(),
+		Name:      evt.ObjectNew.GetName(),
+	}})
+}
+func (ch customHandler) Delete(ctx context.Context, evt event.DeleteEvent, q workqueue.TypedRateLimitingInterface[reconcile.Request]) {
+	q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+		Namespace: evt.Object.GetNamespace(),
+		Name:      evt.Object.GetName(),
+	}})
+}
+func (ch customHandler) Generic(ctx context.Context, evt event.GenericEvent, q workqueue.TypedRateLimitingInterface[reconcile.Request]) {
+	q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+		Namespace: evt.Object.GetNamespace(),
+		Name:      evt.Object.GetName(),
+	}})
+}
