@@ -1,3 +1,5 @@
+//go:build e2e
+
 /*
 Copyright 2020 The Flux CD contributors.
 
@@ -29,7 +31,7 @@ import (
 	"testing"
 	"time"
 
-	"github.com/google/go-github/v49/github"
+	"github.com/google/go-github/v71/github"
 	"github.com/gregjones/httpcache"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -200,6 +202,13 @@ var _ = Describe("GitHub Provider", func() {
 		}
 	}
 
+	It("should get the current user", func() {
+		user, err := c.UserRepositories().GetUserLogin(ctx)
+		Expect(err).ToNot(HaveOccurred())
+
+		Expect(user.GetIdentity()).To(Equal(testUser))
+	})
+
 	It("should list the available organizations the user has access to", func() {
 		// Get a list of all organizations the user is part of
 		orgs, err := c.Organizations().List(ctx)
@@ -343,6 +352,16 @@ var _ = Describe("GitHub Provider", func() {
 		Expect(getSpec.Equals(postSpec)).To(BeTrue())
 	})
 
+	It("should fail when creating a repository with wrong UserLogin", func() {
+		repoName := fmt.Sprintf("test-user-repo-creation-%03d", rand.Intn(1000))
+		repoRef := newUserRepoRef("yadda-yadda-yada", repoName)
+
+		_, err := c.UserRepositories().Create(ctx, repoRef, gitprovider.RepositoryInfo{})
+
+		expectedErr := gitprovider.NewErrIncorrectUser(repoRef.GetIdentity())
+		Expect(err).To(MatchError(expectedErr))
+	})
+
 	It("should error at creation time if the repo already does exist", func() {
 		repoRef := newOrgRepoRef(testOrgName, testOrgRepoName)
 		_, err := c.OrgRepositories().Create(ctx, repoRef, gitprovider.RepositoryInfo{})
