@@ -1,3 +1,5 @@
+//go:build e2e
+
 package gitlab_test
 
 import (
@@ -7,13 +9,13 @@ import (
 
 	"github.com/fluxcd/go-git-providers/gitlab"
 	"github.com/fluxcd/go-git-providers/gitprovider"
-	gogitlab "github.com/xanzy/go-gitlab"
+	gogitlab "gitlab.com/gitlab-org/api/client-go"
 )
 
 func ExampleOrgRepositoriesClient_Get() {
 	// Create a new client
 	ctx := context.Background()
-	c, err := gitlab.NewClient(os.Getenv("GITLAB_TOKEN"), "")
+	c, err := gitlab.NewClient(os.Getenv("GITLAB_ACCESS_TOKEN"), "")
 	checkErr(err)
 
 	// Parse the URL into an OrgRepositoryRef
@@ -26,7 +28,7 @@ func ExampleOrgRepositoriesClient_Get() {
 
 	// Use .Get() to aquire a high-level gitprovider.OrganizationInfo struct
 	repoInfo := repo.Get()
-	// Cast the internal object to a *gogithub.Repository to access custom data
+	// Cast the internal object to a *gogitlab.Project to access custom data
 	internalRepo := repo.APIObject().(*gogitlab.Project)
 
 	fmt.Printf("Description: %s. Homepage: %s", *repoInfo.Description, internalRepo.HTTPURLToRepo)
