@@ -1,3 +1,5 @@
+//go:build e2e
+
 /*
 Copyright 2020 The Flux CD contributors.
 
@@ -35,20 +37,15 @@ import (
 	"github.com/gregjones/httpcache"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
-	"github.com/xanzy/go-gitlab"
+	"gitlab.com/gitlab-org/api/client-go"
 
 	"github.com/fluxcd/go-git-providers/gitprovider"
-	testutils "github.com/fluxcd/go-git-providers/gitprovider/testutils"
+	"github.com/fluxcd/go-git-providers/gitprovider/testutils"
 )
 
 const (
 	ghTokenFile = "/tmp/gitlab-token"
 
-	// Include scheme if custom, e.g.:
-	// gitlabDomain = "https://gitlab.acme.org/"
-	// gitlabDomain = "https://gitlab.dev.wkp.weave.works"
-	gitlabDomain = "gitlab.com"
-
 	defaultDescription = "Foo description"
 	defaultBranch      = "main"
 )
@@ -174,6 +171,11 @@ var _ = Describe("GitLab Provider", func() {
 		ctx context.Context = context.Background()
 		c   gitprovider.Client
 
+		// Include scheme if custom, e.g.:
+		// gitlabDomain = "https://gitlab.acme.org/"
+		// gitlabDomain = "https://gitlab.dev.wkp.weave.works"
+		testBaseUrl string = "gitlab.com"
+
 		// Should exist in environment
 		testOrgName      string = "fluxcd-testing"
 		testSubgroupName string = "fluxcd-testing-sub-group"
@@ -185,6 +187,28 @@ var _ = Describe("GitLab Provider", func() {
 		testOrgRepoName       string = "test-org-repo"
 		testRepoName          string = "test-repo"
 		testTreeRepoName      string = "test-repo-tree"
+
+		orgReposToCleanup []gitprovider.OrgRepositoryRef
+
+		createRepo = func(ctx context.Context, repoRef gitprovider.OrgRepositoryRef) gitprovider.OrgRepository {
+			_, err := c.OrgRepositories().Get(ctx, repoRef)
+			Expect(errors.Is(err, gitprovider.ErrNotFound)).To(BeTrue())
+
+			// Create a new repo
+			_, err = c.OrgRepositories().Create(ctx, repoRef, gitprovider.RepositoryInfo{
+				Description: gitprovider.StringVar(defaultDescription),
+			}, &gitprovider.RepositoryCreateOptions{
+				AutoInit:        gitprovider.BoolVar(true),
+				LicenseTemplate: gitprovider.LicenseTemplateVar(gitprovider.LicenseTemplateApache2),
+			})
+			Expect(err).ToNot(HaveOccurred())
+			orgReposToCleanup = append(orgReposToCleanup, repoRef)
+
+			orgRepo, err := c.OrgRepositories().Get(ctx, repoRef)
+			Expect(err).ToNot(HaveOccurred())
+
+			return orgRepo
+		}
 	)
 
 	BeforeSuite(func() {
@@ -198,6 +222,10 @@ var _ = Describe("GitLab Provider", func() {
 			}
 		}
 
+		if baseUrl := os.Getenv("GITLAB_BASE_URL"); len(baseUrl) != 0 {
+			testBaseUrl = baseUrl
+		}
+
 		if orgName := os.Getenv("GIT_PROVIDER_ORGANIZATION"); len(orgName) != 0 {
 			testOrgName = orgName
 		}
@@ -217,7 +245,7 @@ var _ = Describe("GitLab Provider", func() {
 		var err error
 		c, err = NewClient(
 			gitlabToken, "",
-			gitprovider.WithDomain(gitlabDomain),
+			gitprovider.WithDomain(testBaseUrl),
 			gitprovider.WithDestructiveAPICalls(true),
 			gitprovider.WithConditionalRequests(true),
 			gitprovider.WithPreChainTransportHook(customTransportFactory),
@@ -260,7 +288,7 @@ var _ = Describe("GitLab Provider", func() {
 
 	cleanupOrgRepos := func(prefix string) {
 		fmt.Fprintf(os.Stderr, "Deleting repos starting with %s in org: %s\n", prefix, testOrgName)
-		repos, err := c.OrgRepositories().List(ctx, newOrgRef(testOrgName))
+		repos, err := c.OrgRepositories().List(ctx, newOrgRef(testBaseUrl, testOrgName))
 		Expect(err).ToNot(HaveOccurred())
 		for _, repo := range repos {
 			// Delete the test org repo used
@@ -276,7 +304,7 @@ var _ = Describe("GitLab Provider", func() {
 
 	cleanupUserRepos := func(prefix string) {
 		fmt.Fprintf(os.Stderr, "Deleting repos starting with %s for user: %s\n", prefix, testUserName)
-		repos, err := c.UserRepositories().List(ctx, newUserRef(testUserName))
+		repos, err := c.UserRepositories().List(ctx, newUserRef(testBaseUrl, testUserName))
 		Expect(err).ToNot(HaveOccurred())
 		for _, repo := range repos {
 			// Delete the test org repo used
@@ -290,6 +318,13 @@ var _ = Describe("GitLab Provider", func() {
 		}
 	}
 
+	It("should get the current user", func() {
+		user, err := c.UserRepositories().GetUserLogin(ctx)
+		Expect(err).ToNot(HaveOccurred())
+
+		Expect(user.GetIdentity()).To(Equal(testUserName))
+	})
+
 	It("should list the available organizations the user has access to", func() {
 		// Get a list of all organizations the user is part of
 		orgs, err := c.Organizations().List(ctx)
@@ -336,12 +371,12 @@ var _ = Describe("GitLab Provider", func() {
 			Expect(err).ToNot(HaveOccurred())
 			Expect(getOrg2).ToNot(BeNil())
 		})
-		Expect(hits).To(Equal(1))
+		Expect(hits).To(Equal(1), "unexpected number of cache hits")
 	})
 
 	It("should not fail when .Children is called", func() {
 		_, err := c.Organizations().Children(ctx, gitprovider.OrganizationRef{
-			Domain:       gitlabDomain,
+			Domain:       testBaseUrl,
 			Organization: testOrgName,
 		})
 		Expect(err).ToNot(HaveOccurred())
@@ -349,7 +384,7 @@ var _ = Describe("GitLab Provider", func() {
 
 	It("should be possible to create a group project", func() {
 		// First, check what repositories are available
-		repos, err := c.OrgRepositories().List(ctx, newOrgRef(testOrgName))
+		repos, err := c.OrgRepositories().List(ctx, newOrgRef(testBaseUrl, testOrgName))
 		Expect(err).ToNot(HaveOccurred())
 
 		// Generate a repository name which doesn't exist already
@@ -360,7 +395,7 @@ var _ = Describe("GitLab Provider", func() {
 
 		// We know that a repo with this name doesn't exist in the organization, let's verify we get an
 		// ErrNotFound
-		repoRef := newOrgRepoRef(testOrgName, testOrgRepoName)
+		repoRef := newOrgRepoRef(testBaseUrl, testOrgName, testOrgRepoName)
 		sshURL := repoRef.GetCloneURL(gitprovider.TransportTypeSSH)
 		Expect(sshURL).NotTo(Equal(""))
 		_, err = c.OrgRepositories().Get(ctx, repoRef)
@@ -388,13 +423,13 @@ var _ = Describe("GitLab Provider", func() {
 	})
 
 	It("should error at creation time if the org repo already does exist", func() {
-		repoRef := newOrgRepoRef(testOrgName, testOrgRepoName)
+		repoRef := newOrgRepoRef(testBaseUrl, testOrgName, testOrgRepoName)
 		_, err := c.OrgRepositories().Create(ctx, repoRef, gitprovider.RepositoryInfo{})
 		Expect(errors.Is(err, gitprovider.ErrAlreadyExists)).To(BeTrue())
 	})
 
 	It("should update if the org repo already exists when reconciling", func() {
-		repoRef := newOrgRepoRef(testOrgName, testOrgRepoName)
+		repoRef := newOrgRepoRef(testBaseUrl, testOrgName, testOrgRepoName)
 		// No-op reconcile
 		resp, actionTaken, err := c.OrgRepositories().Reconcile(ctx, repoRef, gitprovider.RepositoryInfo{
 			Description:   gitprovider.StringVar(defaultDescription),
@@ -444,7 +479,7 @@ var _ = Describe("GitLab Provider", func() {
 	It("should update teams with access and permissions when reconciling", func() {
 
 		// Get the test organization
-		orgRef := newOrgRef(testOrgName)
+		orgRef := newOrgRef(testBaseUrl, testOrgName)
 		testOrg, err := c.Organizations().Get(ctx, orgRef)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -455,7 +490,7 @@ var _ = Describe("GitLab Provider", func() {
 		Expect(len(teams)).To(Equal(1), "The 1 team wasn't there...")
 
 		// First, check what repositories are available
-		repos, err := c.OrgRepositories().List(ctx, newOrgRef(testOrgName))
+		repos, err := c.OrgRepositories().List(ctx, newOrgRef(testBaseUrl, testOrgName))
 		Expect(err).ToNot(HaveOccurred())
 
 		// Generate an org repo name which doesn't exist already
@@ -466,7 +501,7 @@ var _ = Describe("GitLab Provider", func() {
 
 		// We know that a repo with this name doesn't exist in the organization, let's verify we get an
 		// ErrNotFound
-		repoRef := newOrgRepoRef(testOrgName, testSharedOrgRepoName)
+		repoRef := newOrgRepoRef(testBaseUrl, testOrgName, testSharedOrgRepoName)
 		_, err = c.OrgRepositories().Get(ctx, repoRef)
 		Expect(errors.Is(err, gitprovider.ErrNotFound)).To(BeTrue())
 
@@ -556,7 +591,7 @@ var _ = Describe("GitLab Provider", func() {
 
 	It("should create, delete and reconcile deploy keys", func() {
 		testDeployKeyName := "test-deploy-key"
-		repoRef := newOrgRepoRef(testOrgName, testSharedOrgRepoName)
+		repoRef := newOrgRepoRef(testBaseUrl, testOrgName, testSharedOrgRepoName)
 
 		orgRepo, err := c.OrgRepositories().Get(ctx, repoRef)
 		Expect(err).ToNot(HaveOccurred())
@@ -566,7 +601,7 @@ var _ = Describe("GitLab Provider", func() {
 		Expect(err).ToNot(HaveOccurred())
 		Expect(len(keys)).To(Equal(0))
 
-		rsaGen := testutils.NewRSAGenerator(256)
+		rsaGen := testutils.NewRSAGenerator(1024)
 		keyPair1, err := rsaGen.Generate()
 		Expect(err).ToNot(HaveOccurred())
 		pubKey := keyPair1.PublicKey
@@ -627,9 +662,73 @@ var _ = Describe("GitLab Provider", func() {
 		}
 	})
 
+	It("should create, delete and reconcile deploy tokens", func() {
+		orgRepo := createRepo(ctx, newOrgRepoRef(
+			testBaseUrl,
+			testOrgName,
+			fmt.Sprintf("deploy-key-spec-repo-%03d", rand.Intn(1000))))
+		testDeployTokenName := "test-deploy-token"
+
+		dts, err := orgRepo.DeployTokens()
+		Expect(err).ToNot(HaveOccurred())
+
+		// List tokens should return 0
+		tokens, err := dts.List(ctx)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(len(tokens)).To(Equal(0))
+
+		testDeployTokenInfo := gitprovider.DeployTokenInfo{
+			Name: testDeployTokenName,
+		}
+		_, err = dts.Create(ctx, testDeployTokenInfo)
+		Expect(err).ToNot(HaveOccurred())
+
+		// List tokens should now return 1
+		tokens, err = dts.List(ctx)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(len(tokens)).To(Equal(1))
+
+		// Getting the token directly should return the same object
+		getToken, err := dts.Get(ctx, testDeployTokenName)
+		Expect(err).ToNot(HaveOccurred())
+
+		Expect(getToken.Get().Name).To(Equal(testDeployTokenInfo.Name))
+
+		// We always have to re-create the token, because we don't know its actual token value
+		Expect(getToken.Set(getToken.Get())).ToNot(HaveOccurred())
+		actionTaken, err := getToken.Reconcile(ctx)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(actionTaken).To(BeTrue())
+
+		// Reconcile creates a new token if the name and username is different
+		name := "new-name"
+		anotherUsername := "another-username"
+		req := getToken.Get()
+		req.Name = name
+		req.Username = anotherUsername
+
+		Expect(getToken.Set(req)).ToNot(HaveOccurred())
+		actionTaken, err = getToken.Reconcile(ctx)
+		// Expect the update to succeed, and modify the state
+		Expect(err).ToNot(HaveOccurred())
+		Expect(actionTaken).To(BeTrue())
+
+		getToken, err = dts.Get(ctx, name)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(getToken.Get().Name).To(Equal(name))
+
+		// Delete the tokens
+		tokens, err = dts.List(ctx)
+		Expect(err).ToNot(HaveOccurred())
+		for _, token := range tokens {
+			err = token.Delete(ctx)
+			Expect(err).ToNot(HaveOccurred())
+		}
+	})
+
 	It("should be possible to create a user project", func() {
 		// First, check what repositories are available
-		repos, err := c.UserRepositories().List(ctx, newUserRef(testUserName))
+		repos, err := c.UserRepositories().List(ctx, newUserRef(testBaseUrl, testUserName))
 		Expect(err).ToNot(HaveOccurred())
 
 		// Generate a repository name which doesn't exist already
@@ -640,7 +739,7 @@ var _ = Describe("GitLab Provider", func() {
 
 		// We know that a repo with this name doesn't exist in the organization, let's verify we get an
 		// ErrNotFound
-		repoRef := newUserRepoRef(testUserName, testRepoName)
+		repoRef := newUserRepoRef(testBaseUrl, testUserName, testRepoName)
 		_, err = c.UserRepositories().Get(ctx, repoRef)
 		Expect(errors.Is(err, gitprovider.ErrNotFound)).To(BeTrue())
 		db := defaultBranchName
@@ -699,13 +798,23 @@ var _ = Describe("GitLab Provider", func() {
 	})
 
 	It("should error at creation time if the user repo already does exist", func() {
-		repoRef := newUserRepoRef(testUserName, testRepoName)
+		repoRef := newUserRepoRef(testBaseUrl, testUserName, testRepoName)
 		_, err := c.UserRepositories().Create(ctx, repoRef, gitprovider.RepositoryInfo{})
 		Expect(errors.Is(err, gitprovider.ErrAlreadyExists)).To(BeTrue())
 	})
 
+	It("should fail info when creating a repository with wrong UserLogin", func() {
+		repoName := fmt.Sprintf("test-user-repo-creation-%03d", rand.Intn(1000))
+		repoRef := newUserRepoRef(testBaseUrl, "yadda-yadda-yada", repoName)
+
+		_, err := c.UserRepositories().Create(ctx, repoRef, gitprovider.RepositoryInfo{})
+
+		expectedErr := gitprovider.NewErrIncorrectUser(repoRef.GetIdentity())
+		Expect(err).To(MatchError(expectedErr))
+	})
+
 	It("should update if the user repo already exists when reconciling", func() {
-		repoRef := newUserRepoRef(testUserName, testRepoName)
+		repoRef := newUserRepoRef(testBaseUrl, testUserName, testRepoName)
 		// No-op reconcile
 		resp, actionTaken, err := c.UserRepositories().Reconcile(ctx, repoRef, gitprovider.RepositoryInfo{
 			Description:   gitprovider.StringVar(defaultDescription),
@@ -756,7 +865,7 @@ var _ = Describe("GitLab Provider", func() {
 	It("should be possible to create and edit a pr for a user repository", func() {
 
 		testRepoName = fmt.Sprintf("test-repo2-%03d", rand.Intn(1000))
-		repoRef := newUserRepoRef(testUserName, testRepoName)
+		repoRef := newUserRepoRef(testBaseUrl, testUserName, testRepoName)
 
 		defaultBranch := defaultBranchName
 
@@ -900,7 +1009,7 @@ var _ = Describe("GitLab Provider", func() {
 
 	It("should be possible to download files from path and branch specified", func() {
 		testTreeRepoName = fmt.Sprintf("test-repo-tree-%03d", rand.Intn(1000))
-		userRepoRef := newUserRepoRef(testUserName, testTreeRepoName)
+		userRepoRef := newUserRepoRef(testBaseUrl, testUserName, testTreeRepoName)
 		// Create a new repo
 		db := defaultBranchName
 		repo, err := c.UserRepositories().Create(ctx, userRepoRef, gitprovider.RepositoryInfo{
@@ -959,7 +1068,7 @@ var _ = Describe("GitLab Provider", func() {
 	})
 
 	It("should be possible to download files from path and branch specified with nested directory", func() {
-		userRepoRef := newUserRepoRef(testUserName, testTreeRepoName)
+		userRepoRef := newUserRepoRef(testBaseUrl, testUserName, testTreeRepoName)
 
 		userRepo, err := c.UserRepositories().Get(ctx, userRepoRef)
 		Expect(err).ToNot(HaveOccurred())
@@ -1010,7 +1119,7 @@ var _ = Describe("GitLab Provider", func() {
 
 	})
 	It("should be possible list repo tree files", func() {
-		userRepoRef := newUserRepoRef(testUserName, testTreeRepoName)
+		userRepoRef := newUserRepoRef(testBaseUrl, testUserName, testTreeRepoName)
 
 		userRepo, err := c.UserRepositories().Get(ctx, userRepoRef)
 		Expect(err).ToNot(HaveOccurred())
@@ -1068,34 +1177,41 @@ var _ = Describe("GitLab Provider", func() {
 		}
 		// Delete the test repo used
 		fmt.Println("Deleting the user repo: ", testRepoName)
-		repoRef := newUserRepoRef(testUserName, testRepoName)
+		repoRef := newUserRepoRef(testBaseUrl, testUserName, testRepoName)
 		repo, err := c.UserRepositories().Get(ctx, repoRef)
-		if errors.Is(err, gitprovider.ErrNotFound) {
-			return
+		if !errors.Is(err, gitprovider.ErrNotFound) {
+			Expect(err).ToNot(HaveOccurred())
+			Expect(repo.Delete(ctx)).ToNot(HaveOccurred())
 		}
-		Expect(err).ToNot(HaveOccurred())
-		Expect(repo.Delete(ctx)).ToNot(HaveOccurred())
 
 		// Delete the test org repo used
 		fmt.Println("Deleting the org repo: ", testOrgRepoName)
-		orgRepoRef := newOrgRepoRef(testOrgName, testOrgRepoName)
+		orgRepoRef := newOrgRepoRef(testBaseUrl, testOrgName, testOrgRepoName)
 		repo, err = c.OrgRepositories().Get(ctx, orgRepoRef)
-		if errors.Is(err, gitprovider.ErrNotFound) {
-			return
+		if !errors.Is(err, gitprovider.ErrNotFound) {
+			Expect(err).ToNot(HaveOccurred())
+			Expect(repo.Delete(ctx)).ToNot(HaveOccurred())
 		}
-		Expect(err).ToNot(HaveOccurred())
-		Expect(repo.Delete(ctx)).ToNot(HaveOccurred())
 
 		// Delete the test shared org repo used
 		fmt.Println("Deleting the shared org repo: ", testSharedOrgRepoName)
-		sharedOrgRepoRef := newOrgRepoRef(testOrgName, testSharedOrgRepoName)
+		sharedOrgRepoRef := newOrgRepoRef(testBaseUrl, testOrgName, testSharedOrgRepoName)
 		repo, err = c.OrgRepositories().Get(ctx, sharedOrgRepoRef)
-		if errors.Is(err, gitprovider.ErrNotFound) {
-			return
+		if !errors.Is(err, gitprovider.ErrNotFound) {
+			Expect(err).ToNot(HaveOccurred())
+			Expect(repo.Delete(ctx)).ToNot(HaveOccurred())
+		}
+
+		for _, ref := range orgReposToCleanup {
+			repo, err = c.OrgRepositories().Get(ctx, ref)
+			if errors.Is(err, gitprovider.ErrNotFound) {
+				continue
+			}
+			Expect(err).ToNot(HaveOccurred())
+			Expect(repo.Delete(ctx)).ToNot(HaveOccurred(), "failed to delete repo %s", ref)
 		}
-		Expect(err).ToNot(HaveOccurred())
-		Expect(repo.Delete(ctx)).ToNot(HaveOccurred())
 	})
+
 })
 
 func expectPRToBeMerged(ctx context.Context, userRepo gitprovider.UserRepository, prNumber int) {
@@ -1106,30 +1222,30 @@ func expectPRToBeMerged(ctx context.Context, userRepo gitprovider.UserRepository
 	}, time.Second*5).Should(BeTrue(), `PR status didn't change to "merged"`)
 }
 
-func newOrgRef(organizationName string) gitprovider.OrganizationRef {
+func newOrgRef(baseUrl, organizationName string) gitprovider.OrganizationRef {
 	return gitprovider.OrganizationRef{
-		Domain:       gitlabDomain,
+		Domain:       baseUrl,
 		Organization: organizationName,
 	}
 }
 
-func newOrgRepoRef(organizationName, repoName string) gitprovider.OrgRepositoryRef {
+func newOrgRepoRef(baseUrl, organizationName, repoName string) gitprovider.OrgRepositoryRef {
 	return gitprovider.OrgRepositoryRef{
-		OrganizationRef: newOrgRef(organizationName),
+		OrganizationRef: newOrgRef(baseUrl, organizationName),
 		RepositoryName:  repoName,
 	}
 }
 
-func newUserRef(userLogin string) gitprovider.UserRef {
+func newUserRef(baseUrl, userLogin string) gitprovider.UserRef {
 	return gitprovider.UserRef{
-		Domain:    gitlabDomain,
+		Domain:    baseUrl,
 		UserLogin: userLogin,
 	}
 }
 
-func newUserRepoRef(userLogin, repoName string) gitprovider.UserRepositoryRef {
+func newUserRepoRef(baseUrl, userLogin, repoName string) gitprovider.UserRepositoryRef {
 	return gitprovider.UserRepositoryRef{
-		UserRef:        newUserRef(userLogin),
+		UserRef:        newUserRef(baseUrl, userLogin),
 		RepositoryName: repoName,
 	}
 }
