@@ -18,6 +18,7 @@ package kustomize
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"os"
@@ -26,7 +27,6 @@ import (
 	"strings"
 	"sync"
 
-	securefs "github.com/fluxcd/pkg/kustomize/filesys"
 	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -39,17 +39,21 @@ import (
 	"sigs.k8s.io/yaml"
 
 	"github.com/fluxcd/pkg/apis/kustomize"
-	"github.com/hashicorp/go-multierror"
+	securefs "github.com/fluxcd/pkg/kustomize/filesys"
 )
 
 const (
 	specField            = "spec"
 	targetNSField        = "targetNamespace"
+	resourcesField       = "resources"
 	patchesField         = "patches"
 	componentsField      = "components"
+	crdsField            = "crds"
 	patchesSMField       = "patchesStrategicMerge"
 	patchesJson6902Field = "patchesJson6902"
 	imagesField          = "images"
+	namePrefixField      = "namePrefix"
+	nameSuffixField      = "nameSuffix"
 )
 
 // Action is the action that was taken on the kustomization file
@@ -67,6 +71,8 @@ const (
 // - a directory path and a kustomization object
 type Generator struct {
 	root          string
+	ignore        string
+	filter        bool
 	kustomization unstructured.Unstructured
 }
 
@@ -83,6 +89,18 @@ func NewGenerator(root string, kustomization unstructured.Unstructured) *Generat
 	}
 }
 
+// NewGeneratorWithIgnore creates a new kustomize generator
+// It takes a root directory, a kustomization object and a string of files to ignore
+// The generator will combine the ignore files with the default ignore files i.e. .sourceignoreÒ
+func NewGeneratorWithIgnore(root, ignore string, kustomization unstructured.Unstructured) *Generator {
+	return &Generator{
+		root:          root,
+		ignore:        ignore,
+		filter:        true,
+		kustomization: kustomization,
+	}
+}
+
 // WithSaveOriginalKustomization will save the original kustomization file
 func WithSaveOriginalKustomization() SavingOptions {
 	return func(dirPath, kfile string, action Action) error {
@@ -133,6 +151,21 @@ func (g *Generator) WriteFile(dirPath string, opts ...SavingOptions) (Action, er
 		return action, fmt.Errorf("%v %v", err, errf)
 	}
 
+	if action == UnchangedAction && len(kus.Resources) == 0 {
+		// if there are no resources, set the build metadata
+		// to avoid "kustomization.yaml is empty" build error
+		kus.BuildMetadata = []string{"originAnnotations"}
+	}
+
+	// apply filters if any
+	if g.filter {
+		err = filterKsWithIgnoreFiles(&kus, dirPath, g.ignore)
+		if err != nil {
+			errf := CleanDirectory(dirPath, action)
+			return action, fmt.Errorf("%v %v", err, errf)
+		}
+	}
+
 	tg, ok, err := g.getNestedString(specField, targetNSField)
 	if err != nil {
 		errf := CleanDirectory(dirPath, action)
@@ -142,6 +175,24 @@ func (g *Generator) WriteFile(dirPath string, opts ...SavingOptions) (Action, er
 		kus.Namespace = tg
 	}
 
+	nprefix, ok, err := g.getNestedString(specField, namePrefixField)
+	if err != nil {
+		errf := CleanDirectory(dirPath, action)
+		return action, fmt.Errorf("%v %v", err, errf)
+	}
+	if ok {
+		kus.NamePrefix = nprefix
+	}
+
+	nsuffix, ok, err := g.getNestedString(specField, nameSuffixField)
+	if err != nil {
+		errf := CleanDirectory(dirPath, action)
+		return action, fmt.Errorf("%v %v", err, errf)
+	}
+	if ok {
+		kus.NameSuffix = nsuffix
+	}
+
 	patches, err := g.getPatches()
 	if err != nil {
 		errf := CleanDirectory(dirPath, action)
@@ -151,7 +202,7 @@ func (g *Generator) WriteFile(dirPath string, opts ...SavingOptions) (Action, er
 	for _, p := range patches {
 		kus.Patches = append(kus.Patches, kustypes.Patch{
 			Patch:  p.Patch,
-			Target: adaptSelector(&p.Target),
+			Target: adaptSelector(p.Target),
 		})
 	}
 
@@ -251,12 +302,12 @@ func (g *Generator) getPatches() ([]kustomize.Patch, error) {
 			patch, ok := p.(map[string]interface{})
 			if !ok {
 				err := fmt.Errorf("unable to convert patch %d to map[string]interface{}", k)
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			var kpatch kustomize.Patch
 			err = runtime.DefaultUnstructuredConverter.FromUnstructured(patch, &kpatch)
 			if err != nil {
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			res = append(res, kpatch)
 		}
@@ -280,12 +331,12 @@ func (g *Generator) getPatchesStrategicMerge() ([]apiextensionsv1.JSON, error) {
 			patch, ok := p.(map[string]interface{})
 			if !ok {
 				err := fmt.Errorf("unable to convert patch %d to map[string]interface{}", k)
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			var kpatch apiextensionsv1.JSON
 			err = runtime.DefaultUnstructuredConverter.FromUnstructured(patch, &kpatch)
 			if err != nil {
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			res = append(res, kpatch)
 		}
@@ -309,12 +360,12 @@ func (g *Generator) getPatchesJson6902() ([]kustomize.JSON6902Patch, error) {
 			patch, ok := p.(map[string]interface{})
 			if !ok {
 				err := fmt.Errorf("unable to convert patch %d to map[string]interface{}", k)
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			var kpatch kustomize.JSON6902Patch
 			err = runtime.DefaultUnstructuredConverter.FromUnstructured(patch, &kpatch)
 			if err != nil {
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			res = append(res, kpatch)
 		}
@@ -338,12 +389,12 @@ func (g *Generator) getImages() ([]kustomize.Image, error) {
 			im, ok := i.(map[string]interface{})
 			if !ok {
 				err := fmt.Errorf("unable to convert patch %d to map[string]interface{}", k)
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			var image kustomize.Image
 			err = runtime.DefaultUnstructuredConverter.FromUnstructured(im, &image)
 			if err != nil {
-				resultErr = multierror.Append(resultErr, err)
+				resultErr = errors.Join(resultErr, err)
 			}
 			res = append(res, image)
 		}
@@ -444,7 +495,14 @@ func (g *Generator) generateKustomization(dirPath string) (Action, string, error
 		resources = append(resources, strings.Replace(file, abs, ".", 1))
 	}
 
-	kus.Resources = resources
+	if len(resources) == 0 {
+		// if there are no resources, set a placeholder namespace
+		// to avoid "kustomization.yaml is empty" build error
+		kus.Namespace = "_placeholder"
+	} else {
+		kus.Resources = resources
+	}
+
 	kd, err := yaml.Marshal(kus)
 	if err != nil {
 		// delete the kustomization file
