@@ -1,6 +1,7 @@
 package internal_test
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"reflect"
@@ -10,7 +11,6 @@ import (
 
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-	"golang.org/x/net/context"
 )
 
 type quickMatcher struct {
@@ -30,7 +30,7 @@ func (q quickMatcher) NegatedFailureMessage(actual any) (message string) {
 	return fmt.Sprintf("QM negated failure message: %v", actual)
 }
 
-func (q quickMatcher) MatchMayChangeInTheFuture(actual interface{}) bool {
+func (q quickMatcher) MatchMayChangeInTheFuture(actual any) bool {
 	if q.oracleFunc == nil {
 		return true
 	}
@@ -180,10 +180,11 @@ var _ = Describe("Asynchronous Assertions", func() {
 				Ω(ig.RegisteredHelpers).Should(ContainElement("(*AsyncAssertion).match"))
 			})
 
-			It("renders the matcher's error if an error occured", func() {
+			It("renders the matcher's error if an error occurred", func() {
 				ig.G.Eventually(ERR_MATCH).WithTimeout(50 * time.Millisecond).WithPolling(10 * time.Millisecond).Should(SpecMatch())
 				Ω(ig.FailureMessage).Should(ContainSubstring("Timed out after"))
-				Ω(ig.FailureMessage).Should(ContainSubstring("The matcher passed to Eventually returned the following error:\nspec matcher error"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("The matcher passed to Eventually returned the following error:"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("spec matcher error"))
 			})
 
 			It("renders the optional description", func() {
@@ -274,6 +275,22 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(ig.FailureMessage).Should(ContainSubstring("Context was cancelled after"))
 					Ω(ig.FailureMessage).Should(ContainSubstring("There is no failure as the matcher passed to Consistently has not yet failed"))
 				})
+
+				It("includes the cancel cause if provided", func() {
+					ctx, cancel := context.WithCancelCause(context.Background())
+					counter := 0
+					ig.G.Eventually(func() string {
+						counter++
+						if counter == 2 {
+							cancel(fmt.Errorf("kaboom"))
+						} else if counter == 10 {
+							return MATCH
+						}
+						return NO_MATCH
+					}, time.Hour, ctx).Should(SpecMatch())
+					Ω(ig.FailureMessage).Should(ContainSubstring("Context was cancelled (cause: kaboom) after"))
+					Ω(ig.FailureMessage).Should(ContainSubstring("positive: no match"))
+				})
 			})
 
 			Context("when the passed-in context is a Ginkgo SpecContext that can take a progress reporter attachment", func() {
@@ -329,6 +346,26 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(ig.FailureMessage).Should(ContainSubstring("Context was cancelled after"))
 				})
 
+				It("uses the default timeout if the user explicitly opts into EnforceDefaultTimeoutsWhenUsingContexts()", func() {
+					ig.G.SetDefaultEventuallyTimeout(time.Millisecond * 100)
+					ig.G.SetDefaultEventuallyPollingInterval(time.Millisecond * 10)
+					ig.G.EnforceDefaultTimeoutsWhenUsingContexts()
+					t := time.Now()
+					ctx, cancel := context.WithCancel(context.Background())
+					iterations := 0
+					ig.G.Eventually(func() string {
+						iterations += 1
+						if time.Since(t) > time.Millisecond*1000 {
+							cancel()
+						}
+						return "A"
+					}).WithContext(ctx).Should(Equal("B"))
+					Ω(time.Since(t)).Should(BeNumerically("~", time.Millisecond*100, time.Millisecond*50))
+					Ω(iterations).Should(BeNumerically("~", 100/10, 2))
+					Ω(ig.FailureMessage).Should(ContainSubstring("Timed out after"))
+					Ω(ctx.Err()).Should(BeNil())
+				})
+
 				It("uses the explicit timeout when it is provided", func() {
 					t := time.Now()
 					ctx, cancel := context.WithCancel(context.Background())
@@ -372,7 +409,8 @@ var _ = Describe("Asynchronous Assertions", func() {
 				}).WithTimeout(50 * time.Millisecond).WithPolling(10 * time.Millisecond).Should(SpecMatch())
 				Ω(counter).Should(Equal(3))
 				Ω(ig.FailureMessage).Should(ContainSubstring("Failed after"))
-				Ω(ig.FailureMessage).Should(ContainSubstring("The matcher passed to Consistently returned the following error:\nspec matcher error"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("The matcher passed to Consistently returned the following error:"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("spec matcher error"))
 			})
 
 			It("fails if the matcher doesn't match at any point", func() {
@@ -441,10 +479,11 @@ var _ = Describe("Asynchronous Assertions", func() {
 				Ω(ig.RegisteredHelpers).Should(ContainElement("(*AsyncAssertion).match"))
 			})
 
-			It("renders the matcher's error if an error occured", func() {
+			It("renders the matcher's error if an error occurred", func() {
 				ig.G.Consistently(ERR_MATCH).Should(SpecMatch())
 				Ω(ig.FailureMessage).Should(ContainSubstring("Failed after"))
-				Ω(ig.FailureMessage).Should(ContainSubstring("The matcher passed to Consistently returned the following error:\nspec matcher error"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("The matcher passed to Consistently returned the following error:"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("spec matcher error"))
 			})
 
 			It("renders the optional description", func() {
@@ -588,7 +627,8 @@ var _ = Describe("Asynchronous Assertions", func() {
 					ig.G.Eventually(func() (int, string, Foo, error) {
 						return 1, "", Foo{}, errors.New("welp!")
 					}).WithTimeout(50 * time.Millisecond).WithPolling(10 * time.Millisecond).Should(BeNumerically("<", 100))
-					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually returned the following error:\nwelp!"))
+					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually returned the following error:"))
+					Ω(ig.FailureMessage).Should(ContainSubstring("welp!"))
 				})
 
 				Context("when making a ShouldNot assertion", func() {
@@ -716,7 +756,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 				It("correctly handles the case (in concert with Ginkgo) when an assertion fails in a goroutine", func() {
 					count := 0
 					ig.G.Eventually(func(g Gomega) {
-						c := make(chan interface{})
+						c := make(chan any)
 						go func() {
 							defer GinkgoRecover()
 							defer close(c)
@@ -763,7 +803,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(ig.FailureMessage).Should(ContainSubstring("Expected\n    <bool>: false\nto be true"))
 				})
 
-				It("shows the state of the last match if there was a non-failing funciton at some point", func() {
+				It("shows the state of the last match if there was a non-failing function at some point", func() {
 					counter := 0
 					_, file, line, _ := runtime.Caller(0)
 					ig.G.Eventually(func(g Gomega) int {
@@ -877,7 +917,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(ig.FailureMessage).ShouldNot(ContainSubstring("bloop"))
 				})
 
-				It("returns the first failed assertion as an error and should satisy ShouldNot(Succeed) eventually", func() {
+				It("returns the first failed assertion as an error and should satisfy ShouldNot(Succeed) eventually", func() {
 					counter := 0
 					ig.G.Eventually(func(g Gomega) {
 						counter += 1
@@ -927,7 +967,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(counter).Should(Equal(3))
 				})
 
-				It("returns the first failed assertion as an error and should satisy ShouldNot(Succeed) consistently if an error always occur", func() {
+				It("returns the first failed assertion as an error and should satisfy ShouldNot(Succeed) consistently if an error always occur", func() {
 					counter := 0
 					ig.G.Consistently(func(g Gomega) {
 						counter += 1
@@ -993,7 +1033,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 				})
 			})
 
-			Context("with a Gomega arugment as well", func() {
+			Context("with a Gomega argument as well", func() {
 				It("can also forward arguments alongside a Gomega", func() {
 					Eventually(func(g Gomega, a int, b int) {
 						g.Expect(a).To(Equal(b))
@@ -1004,7 +1044,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 				})
 			})
 
-			Context("with a context arugment as well", func() {
+			Context("with a context argument as well", func() {
 				It("can also forward arguments alongside a context", func() {
 					ctx := context.WithValue(context.Background(), "key", "value")
 					Eventually(func(ctx context.Context, animal string) string {
@@ -1013,7 +1053,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 				})
 			})
 
-			Context("with Gomega and context arugments", func() {
+			Context("with Gomega and context arguments", func() {
 				It("forwards arguments alongside both", func() {
 					ctx := context.WithValue(context.Background(), "key", "I have")
 					f := func(g Gomega, ctx context.Context, count int, zoo ...string) {
@@ -1042,27 +1082,27 @@ var _ = Describe("Asynchronous Assertions", func() {
 					ig.G.Eventually(func(a int) string {
 						return ""
 					}).Should(Equal("foo"))
-					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int) string takes 1 arguments but 0 have been provided.  Please use Eventually().WithArguments() to pass the corect set of arguments."))
+					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int) string takes 1 arguments but 0 have been provided.  Please use Eventually().WithArguments() to pass the correct set of arguments."))
 
 					ig.G.Eventually(func(a int, b int) string {
 						return ""
 					}).WithArguments(1).Should(Equal("foo"))
-					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int, int) string takes 2 arguments but 1 has been provided.  Please use Eventually().WithArguments() to pass the corect set of arguments."))
+					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int, int) string takes 2 arguments but 1 has been provided.  Please use Eventually().WithArguments() to pass the correct set of arguments."))
 
 					ig.G.Eventually(func(a int, b int) string {
 						return ""
 					}).WithArguments(1, 2, 3).Should(Equal("foo"))
-					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int, int) string takes 2 arguments but 3 have been provided.  Please use Eventually().WithArguments() to pass the corect set of arguments."))
+					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int, int) string takes 2 arguments but 3 have been provided.  Please use Eventually().WithArguments() to pass the correct set of arguments."))
 
 					ig.G.Eventually(func(g Gomega, a int, b int) string {
 						return ""
 					}).WithArguments(1, 2, 3).Should(Equal("foo"))
-					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(types.Gomega, int, int) string takes 3 arguments but 4 have been provided.  Please use Eventually().WithArguments() to pass the corect set of arguments."))
+					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(types.Gomega, int, int) string takes 3 arguments but 4 have been provided.  Please use Eventually().WithArguments() to pass the correct set of arguments."))
 
 					ig.G.Eventually(func(a int, b int, c ...int) string {
 						return ""
 					}).WithArguments(1).Should(Equal("foo"))
-					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int, int, ...int) string takes 3 arguments but 1 has been provided.  Please use Eventually().WithArguments() to pass the corect set of arguments."))
+					Ω(ig.FailureMessage).Should(ContainSubstring("The function passed to Eventually has signature func(int, int, ...int) string takes 3 arguments but 1 has been provided.  Please use Eventually().WithArguments() to pass the correct set of arguments."))
 
 				})
 			})
@@ -1180,6 +1220,19 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(ig.FailureMessage).Should(ContainSubstring("Told to stop trying after"))
 					Ω(ig.FailureMessage).Should(ContainSubstring("bam"))
 				})
+
+				It("fails, even if the match were to happen to succeed and the user uses Succeed", func() {
+					ig.G.Eventually(func() (int, error) {
+						i += 1
+						if i < 3 {
+							return i, nil
+						}
+						return i, StopTrying("bam").Successfully()
+					}).Should(Equal(3))
+					Ω(i).Should(Equal(3))
+					Ω(ig.FailureMessage).Should(ContainSubstring("Told to stop trying (and ignoring call to Successfully(), as it is only relevant with Consistently) after"))
+					Ω(ig.FailureMessage).Should(ContainSubstring("bam"))
+				})
 			})
 
 			Context("when returned as the sole actual", func() {
@@ -1238,7 +1291,7 @@ var _ = Describe("Asynchronous Assertions", func() {
 			})
 
 			Context("when used with consistently", func() {
-				It("always signifies a failure", func() {
+				It("signifies a failure", func() {
 					ig.G.Consistently(func() (int, error) {
 						i += 1
 						if i >= 3 {
@@ -1250,6 +1303,17 @@ var _ = Describe("Asynchronous Assertions", func() {
 					Ω(ig.FailureMessage).Should(ContainSubstring("Told to stop trying after"))
 					Ω(ig.FailureMessage).Should(ContainSubstring("bam"))
 				})
+
+				It("signifies success when called Successfully", func() {
+					Consistently(func() (int, error) {
+						i += 1
+						if i >= 3 {
+							return i, StopTrying("bam").Successfully()
+						}
+						return i, nil
+					}).Should(BeNumerically("<", 10))
+					Ω(i).Should(Equal(3))
+				})
 			})
 
 			Context("when StopTrying has attachments", func() {
@@ -1529,7 +1593,8 @@ sprocket:
 				}).WithTimeout(time.Millisecond*10).Should(QuickMatcher(func(actual any) (bool, error) {
 					return false, fmt.Errorf("matcher-error")
 				}), "My Description")
-				Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe matcher passed to Eventually returned the following error:\nmatcher-error\n    <*errors.errorString"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe matcher passed to Eventually returned the following error:\n    <*errors.errorString"))
+				Ω(ig.FailureMessage).Should(ContainSubstring("matcher-error"))
 			})
 
 			When("the matcher error is a StopTrying with attachments", func() {
@@ -1553,7 +1618,8 @@ sprocket:
 					}).WithTimeout(time.Millisecond*10).Should(QuickMatcher(func(actual any) (bool, error) {
 						return true, nil
 					}), "My Description")
-					Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe function passed to Eventually returned the following error:\nactual-err\n    <*errors.errorString"))
+					Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe function passed to Eventually returned the following error:\n    <*errors.errorString"))
+					Ω(ig.FailureMessage).Should(ContainSubstring("actual-err"))
 				})
 			})
 
@@ -1569,7 +1635,7 @@ sprocket:
 			})
 
 			When("the actual error is because there was an assertion failure in the function, and there are return values", func() {
-				It("emits a clear message about the error having occured", func() {
+				It("emits a clear message about the error having occurred", func() {
 					_, file, line, _ := runtime.Caller(0)
 					ig.G.Eventually(func(g Gomega) int {
 						g.Expect(true).To(BeFalse())
@@ -1608,8 +1674,10 @@ sprocket:
 							}
 							return false, nil
 						}), "My Description")
-						Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe function passed to Eventually returned the following error:\nactual-err\n    <*errors.errorString"))
-						Ω(ig.FailureMessage).Should(ContainSubstring("At one point, however, the function did return successfully.\nYet, Eventually failed because the matcher returned the following error:\nmatcher-err"))
+						Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe function passed to Eventually returned the following error:\n    <*errors.errorString"))
+						Ω(ig.FailureMessage).Should(ContainSubstring("actual-err"))
+						Ω(ig.FailureMessage).Should(ContainSubstring("At one point, however, the function did return successfully.\nYet, Eventually failed because the matcher returned the following error:"))
+						Ω(ig.FailureMessage).Should(ContainSubstring("matcher-err"))
 					})
 				})
 
@@ -1627,7 +1695,8 @@ sprocket:
 							actualInt := actual.(int)
 							return actualInt > 3, nil
 						}), "My Description")
-						Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe function passed to Eventually returned the following error:\nactual-err\n    <*errors.errorString"))
+						Ω(ig.FailureMessage).Should(ContainSubstring("My Description\nThe function passed to Eventually returned the following error:\n    <*errors.errorString"))
+						Ω(ig.FailureMessage).Should(ContainSubstring("actual-err"))
 						Ω(ig.FailureMessage).Should(ContainSubstring("At one point, however, the function did return successfully.\nYet, Eventually failed because the matcher was not satisfied:\nQM failure message: 3"))
 
 					})
